#!/bin/bash
#
# Install New Refresh Version
#
# Michal Augustyniak : michal.augustyniak@cornelius.com
#
# This file must be located in and run from the Scripts subdirectory
# of the version to be installed
# For example: /opt/Refresh/Versions/Refresh-1.6.1/Scripts

NORMAL=0
ERROR=1

# If running on a remote terminal, set display 0 for anything that needs it
setDisplay=""
if [ -z "$DISPLAY" ] ; then
  setDisplay="DISPLAY=:0"
fi

exitWithError() {
  echo "Error: $1"
  exit $ERROR
}

devMode="false"
# process any arguments passed in
for arg in "$@"
do
    if [ "$arg" == "dev" ]; then
      devMode="true"
    fi
done

if [ "$SYSTEM_TYPE" == "" ]; then
  SYSTEM_TYPE=$(</opt/Refresh/systemType.txt)
fi

if [ "$SYSTEM_TYPE" == "" ]; then
    exitWithError "Must set SYSTEM_TYPE before running this script"
fi

# GET THE VERSION NUMBER FROM THE METADATA FILE
[ -f ../metadata.txt ] || exitWithError "metadata.txt file not found"
VERSION=$(awk -F "=" '/version/ {print $2}' ../metadata.txt)

echo "Installing v$VERSION ..."
##############################
# Change to the root of this version
cd ..
##############################

optRefreshDirectory=`pwd`
[ -d $optRefreshDirectory ] || exitWithError "Software version not found"
optRefreshScriptsDirectory="$optRefreshDirectory/Scripts"

# CLEAN OUT COMPILED PYTHON FILES GENERATED WHEN VIDEOPLAYER IS RUN ON TARGET
# IF THESE ARE NOT DELETED THE CHECKSUM VERIFICATION WILL FAIL BECAUSE THESE FILES DID NOT EXIST
# WHEN THE CHECKSUM WAS CALCULATED
# But in 0.7.7 or later, do not delete the file that has the database key ;-)
mv VideoPlayer/pragma.pyc VideoPlayer/pragma.pyc.save
rm -f VideoPlayer/*.pyc || exitWithError "Failed to remove compiled python files"
mv VideoPlayer/pragma.pyc.save VideoPlayer/pragma.pyc

# CLEAN OUT COMPILED PYTHON FILES GENERATED WHEN IoT IS RUN ON TARGET
# IF THESE ARE NOT DELETED THE CHECKSUM VERIFICATION WILL FAIL BECAUSE THESE FILES DID NOT EXIST
# WHEN THE CHECKSUM WAS CALCULATED
# But in 0.7.7 or later, do not delete the file that has the database key ;-)
mv IoT/pragma.pyc IoT/pragma.pyc.save
rm -f IoT/*.pyc || exitWithError "Failed to remove compiled python files"
mv IoT/pragma.pyc.save IoT/pragma.pyc

# VERIFY SOFTWARE CHECKSUM BEFORE CONTINUING
echo "Verifying software integrity"
echo `pwd`
[ -f checksum.md5sum ] || exitWithError "checksum.md5sum is missing"
EXPECTED_CHECKSUM=`cat checksum.md5sum`
# Workload file is generated by Apacer disk utility, ignore it
CALCULATED_CHECKSUM=`find . -type f -not -name "*.md5sum" -not -name "Workload.ini" -exec md5sum {} + | awk '{print $1}' | sort | md5sum`
[ "$EXPECTED_CHECKSUM" == "$CALCULATED_CHECKSUM" ] || exitWithError "Checksums did not match. Expected $EXPECTED_CHECKSUM, Calculated $CALCULATED_CHECKSUM"


# COPY PROJECT ASSETS
echo "Installing Resources, please wait..."

###############################################
cd $optRefreshScriptsDirectory || exitWithError "cd $optRefreshScriptsDirectory failed"
###############################################

cp -arf ../Wallpapers/Refresh.png /opt/Refresh/Wallpapers/. || exitWithError "Wallpaper copy failed"
cp -arf ../Wallpapers/Cornelius-Splash-Screen.tar.gz /opt/Refresh/Wallpapers/. || exitWithError "Splash copy failed"
cp -arf ../Pictures/Shared/*        /opt/Refresh/Pictures/Shared/. || echo "Pictures/Shared skipped"
cp -arf ../Pictures/MainUI/*        /opt/Refresh/Pictures/MainUI/. || exitWithError "Pictures/MainUI copy failed"
cp -arf ../Pictures/ServiceUI/*     /opt/Refresh/Pictures/ServiceUI/. || exitWithError "Pictures/ServiceUI copy failed"
cp -arf ../Videos/*                 /opt/Refresh/Videos/. || echo "Videos skipped"
rm -f /opt/Refresh/Manuals/*
cp -arf ../Manuals/*                /opt/Refresh/Manuals/. || echo "Manuals skipped"

cp -arf ../systemType.txt           /opt/Refresh/. || echo "Missing system type file"

if [ ! -d /opt/Refresh/Videos/CMS ]
then mkdir /opt/Refresh/Videos/CMS
     mkdir /opt/Refresh/Videos/CMS/tmp
fi

if [ ! -d /opt/Refresh/IoT ]
then mkdir /opt/Refresh/IoT
fi

if [ ! -d /opt/Refresh/Updates ]
then mkdir /opt/Refresh/Updates
fi

cp -v ../*.hex /opt/Refresh/Versions
cp -v ../Database/nutrition.csv /opt/Refresh/Database


# INSTALL ANY EXTRA LINUX PACKAGES NEEDED
shopt -s nullglob

# Check if the directory exists and is NOT empty
if [ -d "$optRefreshDirectory/packages" ] && [ -n "$(ls -A $optRefreshDirectory/packages)" ]; then
  
  if [ -e "$optRefreshDirectory/packages/wicd" ]; then
    dpkg -R -G -i $optRefreshDirectory/packages/wicd
  fi

  for f in $optRefreshDirectory/packages/*.deb
  do
     echo "Installing $f..."
     dpkg --force-all -G -i $f
  done

  #shopt -s nullglob
  #for f in $optRefreshDirectory/packages/wicd/*.deb
  #do
  #   echo "Installing $f..."
  #   dpkg --force-all -G -i $f
  #done


  #INSTALL ANY EXTRA PYTHON PACKAGES
  for f in $optRefreshDirectory/packages/python/*.egg
  do
     echo "Installing $f..."
     easy_install $f
  done

  for f in $optRefreshDirectory/packages/python/*.gz
  do
     echo "Installing $f..."
     easy_install $f
  done

  #INSTALL QT binaries
  if [ -e $optRefreshDirectory/packages/qt/Qt5.6.0_bin.tar.gz ]; then
    if [ -e /opt/Qt5.3.1/5.3/Src ]
    then 
       rm -rf /opt/Qt5.3.1/5.3/Src
    fi

    if [ ! -e  /opt/Qt5.6.0 ]
    then
       echo "Installing QT "
       tar xf $optRefreshDirectory/packages/qt/Qt5.6.0_bin.tar.gz -C /opt
    fi

    #install patch to disable touchscreen warnings
    echo "Install QT Touch patch"
    tar xf $optRefreshDirectory/packages/qt/qt_ev_patch.tar.gz -C /opt/Qt5.6.0/5.6/gcc_64/lib

    #install patch for SQL to use USLEEP
    echo "Install QT Sleep patch"
    cp -v  $optRefreshDirectory/packages/qt/libqsqlcipher.so  /opt/Qt5.6.0/5.6/gcc_64/plugins/sqldrivers
  fi

  #install Gstreamer 1.1.12
  if [ -e "$optRefreshDirectory/packages/gst/gst.tar.gz"]; then
     echo "Installing GStreamer "
     tar xf $optRefreshDirectory/packages/gst/gst.tar.gz -C /opt
     rm -rf /usr/lib/python2.7/dist-packages/gi
     tar xf $optRefreshDirectory/packages/gst/gst_py.tar.gz -C /usr/lib/python2.7/dist-packages

    #install new ldconf config so gstreamer is found 
    echo "Updating ld.conf.so"
    cp -v  libc.conf /etc/ld.so.conf.d
    ldconfig 
  fi

  #reset some possible messes up kde config files
  if [ -d "$optRefreshDirectory/packages/kde" ] && [ -d "/home/cornelius/.kde" ]; then
    cp  $optRefreshDirectory/packages/kde/* /home/cornelius/.kde/share/config
  fi

fi  # end IF have packages

echo "Updating dhclient.conf"
cp -v dhclient.conf /etc/dhclient

#INSTALL sqlcipher
cp -r /opt/Refresh/Versions/usr/local/* /usr/local
rm -r /opt/Refresh/Versions/usr

if [ "$devMode" == "true" ] ; then
  echo "SKIPPING log rotate & autostart"
else
  echo "Setting log rotate"
  ./set-logrotate.sh || exitWithError "set-logrotate failed"

  echo "Setting autostart"
  ./set-autostart.sh || exitWithError "set-autostart failed"
fi

# REMOVE SYMBOLIC LINK TO CURRENT VERSION AND REPLACE WITH LINK TO NEW VERSION
echo "Linking current Version"
cd ../.. || exitWithError "cd ../.. failed"
rm -f Refresh-current || exitWithError "rm -f Refresh-current failed"
ln -s $optRefreshDirectory Refresh-current || exitWithError "ln -s Refresh-$VERSION Refresh-current failed"

# REMOVE SYMBOLIC LINK TO CURRENT CORCO TOOL AND REPLACE WITH LINK TO NEW CORCO TOOL
echo "Setting up CorcoTool"
cd /usr/local/bin || exitWithError "cd /usr/local/bin failed"
rm -f corco || exitWithError "rm -f corco failed"
ln -s $optRefreshDirectory/Scripts/corco.sh corco || exitWithError "ln -s $optRefreshDirectory/Scripts/corco.sh corco failed"

if [ "$devMode" == "true" ] ; then
  echo "SKIPPING SUFF..."
else
  # CREATED ENCRYPTED DATABASE IF IT DOES NOT EXIST
  cd $optRefreshScriptsDirectory || exitWithError "cd $optRefreshScriptsDirectory failed"
  if [ ! -e /opt/Refresh/Database/refresh_database_enc.db ]
  then   
    if [ -e /opt/Refresh/Database/refresh_database.db ]
     then 
        echo "Encrypting database"
        python db-enc.py /opt/Refresh/Database/refresh_database.db /opt/Refresh/Database/refresh_database_enc.db
     else 
        echo "No database found to encrypt"
     fi
  fi


  if [  -e /opt/Refresh/Database/refresh_database_enc.db ]
  then 
    python db-fix-midcarb.py /opt/Refresh/Database/refresh_database_enc.db
  fi

  #Replace crontab with an hourly version
  cp -v ./crontab /etc

  if [ "$SYSTEM_TYPE" == 'idc' ]; then
    # edk: I'm not sure what the intent is here, but it certainly isn't reliable because
    # the interface name isn't guaranteed to be consistent.
    FOUND=`grep "eth0:" /proc/net/dev`
    l=$(echo $FOUND | wc -c)
    if [ $l -gt 0 ]
    then
       echo "Network found"
    else
       ./ethfix.sh
    fi

    echo "Update Grub"
    cp grub /etc/default/grub
    update-grub
  fi
  
# end IF not devmode
fi


# Openbox configuration for Viper...
if [ "$SYSTEM_TYPE" == 'viper' ]; then
  # Openbox configuration file
  openboxRcFile="/home/cornelius/.config/openbox/rc.xml"

  # Check if there is a local config
  if [ ! -f $openboxRcFile ] ; then
    # Copy the default file if its missing
    cp /etc/xdg/openbox/rc.xml $openboxRcFile
  fi

  # All of the following modifications to the rc.xml file are presuming that the
  # default file already contains some tags.  This will not work properly
  # for a blank file...

  if [ ! -f $openboxRcFile ] ; then
    echo "Unable to configure openbox"
  else
    # Check if wicd is already configured
    wicdConf=$(grep "wicd-client.py" $openboxRcFile)

    if [ "$wicdConf" == "" ] ; then
      # Insert the new application rule to prevent wicd
      # from falling behind the app (layer = above)
      sed -i 's/<applications>/<applications>\n  <application name=\"wicd-client.py\">\n    <layer>above<\/layer>\n  <\/application>\n/' $openboxRcFile

    fi

    # Reload the new configuration settings
    # Need to use shell to set the display
    sh -c "$setDisplay openbox --reconfigure"
  fi

  # Remove NetworkManager, it conflicts with wicd
  # First have to remove plainbox
  # Autoremove anything that may block apt-get
  # -qq = quiet and auto-accept
  apt-get -qq autoremove 
  apt-get -qq remove plainbox-provider-resource-generic
  apt-get -qq remove network-manager
  apt-get -qq autoremove

  # For some reason, wicd doesn't enable wifi during startup, so have to add this to the init script
  # if it's not already there.  Currently the default script doesn't have this anywhere in it
  # so this check is good enough.  If they ever change the default then this will need to be improved.
  unblockWifi=$(grep -e "rfkill [ ]*unblock [ ]*wifi" /etc/init.d/wicd)

  if [ -z "$unblockWifi" ] ; then
    # Look for the "do_start" function then add rfkill at the start of it
    sed -i '/do_start()/,/}/ {s/{/{\n        rfkill unblock wifi/}' /etc/init.d/wicd
  fi

  # Set up the onscreen keyboard
  # It has to run at least once before these settings to take effect
  # Otherwise it'll copy the default settings during its first startup
  # Have to use sh -c in order to select the display if its not local
  sh -c "$setDisplay onboard &"
  sleep 2
  pkill onboard
  # Once docked, there is no way to undock it because the docks are disabled,
  # so make sure it starts up undocked
  sh -c "$setDisplay gsettings set org.onboard.window docking-enabled false"
  sh -c "$setDisplay gsettings set org.onboard layout 'Full Keyboard'"
  sh -c "$setDisplay gsettings set org.onboard.window.landscape y 700"
  sh -c "$setDisplay gsettings set org.onboard.window.landscape x 70"
  sh -c "$setDisplay gsettings set org.onboard.window.landscape width 1024"
  sh -c "$setDisplay gsettings set org.onboard.window.landscape height 320"

fi

# Remove crash reports and disable apport (crash reporter popup)
echo "Disabling Ubuntu crash reports"
sudo service apport stop
sudo rm -rf /var/crash/*
sudo sed -i -e '/^enabled=/ s/^.*$/enabled=0/' /etc/default/apport

#turn off Ubuntu upgrades
echo "Turn off Ubuntu LTS upgrades"
cp -v  ./release-upgrades /etc/update-manager

if [ "$devMode" == "true" ] ; then
  echo "SKIPPING REBOOT"
else
  [ -f ~/noreboot_on_install ] && echo "NO REBOOT ON INSTALL"
  [ -f ~/noreboot_on_install ] || reboot
fi

